# EGN-EP

Nextflow version of EuGene Eukaryote Pipeline

## INSTALL

### PRE-REQUISITES

- wget
- gzip
- NextFlow (> 21.04.0)  (Java 8 or later is required) : <https://www.nextflow.io/index.html#GetStarted>
- Singularity container app (> 3.0) : <https://sylabs.io/guides/3.0/user-guide/index.html>
- a local copy of the blast/usearch container (see USEARCH section for more details)

## Usage

```bash
./egn-ep.sh --help

#Load mandatory modules/environment (Singularity & NextFlow)

Usage:
./egn-ep.sh [--parameters]  [--advanced] [--usearch] [--slurm] --sequences <path to fasta file> --config <path to EuGene-P config file (protein and ESTs dbs)>

#more details
./egn-ep.sh --parameters

#complete list of parameters 
./egn-ep.sh --advanced

#each parameter can be overwritten using command line syntax (--param) or adding it in config file

# --slurm parameter disables nohup/background submission which is incompatible with sbatch submission
```

## Run

### Proteic Databases

We provide a set of proteic databases ready to use. These databases are built upon publicly available databases (Uniprot/Swiss-Prot and Uniprot/TrEMBL) cleaned from identified repeated elements (match with Dfam consensus sequences <https://dfam.org> and Transposon-PSI transposon protein database <http://transposonpsi.sourceforge.net/>)

These databases are available at <http://eugene.toulouse.inra.fr/Downloads/dbs/>
The following domains are available:

- fungi
- nematodes
- oomycetes
- plant
- athropods

You can download them at once using the helper script:

```bash
./utils/getDbs.sh $DOMAIN
```

Downloaded databases are located in the `./db` directory and a default configuration file `egnep.default.cfg` is produced.

You have to choose the protein databank for training IMM (a reference proteome)

```bash
blastx_db_{W}_training=1
```

### Transcript Datasets

Add a transcriptome dataset (check section Transcript Dataset Cleaning)

1. Add a new EST entry (for instance the entry number X) in the configuration file (default  `egnep.default.cfg`):

```bash
est_{X}_file='database_path'
est_{X}_pcs=30
est_{X}_pci=97
est_{X}_remove_unspliced=0
est_{X}_preserve=1
est_{X}_training=0
est_{X}_priority=0
```

2. complete the transcriptome list 'est_list' with the entry number X:

```bash
est_list='. . . {X}'
```

3. Choose the transcriptome for training IMM
 You have to choose one reference transcriptome, used to train statistical models of coding regions and intronic region

```bash
est_{Z}_training=1
```

#### Transcript Dataset Cleaning

To avoid gmap memory issues or excessive execution time, transcript dataset must be cleaned.
We advise to clean this dataset before running EGN-EP as it should be a time consuming step.
The recommended cleaning steps are the following:

1. **rDNA removal**
2. **polyX trimming** (and internal polyX identification split)

We provide a script that runs these steps using EGN-EP singularity containers (so singularity must be installed):

```bash
$ ./utils/cleanTranscripts.sh 

Usage:
./utils/cleanTranscripts.sh  --kingdom  <euk|arc|bac|mito> --sequences <PATH TO TRANSCRIPT FILE (fasta)> --cpus <CPUS>
```

### Launch

```bash
./egn-ep.sh --sequences ./path/to/genomicSequences.fasta \
 --domain plant|fungi|nematodes|oomycetes|arthropods  \ 
 --config ./path/to/configuration/file.cfg \
 -profile $MYPROFILE 
```

**Beware of single (-) and double dash (--), they involve different nextflow behaviour** 

If you  want to set a new profile to fit with your cluster you have to:

1. Create a new config file: `conf/profile/$MYPROFILE.config`  (please, check `conf/profile/template.config`)
2. Add this new profile name in `nextflow.config` file in *profiles {}*  section:
```groovy
profiles {
  $MYPROFILE {
    includeConfig 'conf/profiles/$MYPROFILE.config'
  }
}
```

### Advanced parameters

1. Change the statistical models for the detection of splice sites

```bash
--domain  values:fungi,nematodes,oomycetes,plant,athropods
```

2. Choose the method for the protein similarity search

```bash
--usearch | --diamond | --blastx
```

Three methods are available:

- `--diamond` *[default]* First perform a protein database reduction with diamond, then launch classical blastx search against the reduced database.
- `--blastx`         Classical blastx search. Very good sensitivity but long calculation time.
- `--usearch`  First perform a protein database reduction with ublast from usearch. (<https://www.drive5.com/usearch/>)
                Then launch classical blastx search against the reduced database
                Good sensitivity and fast calculation time.
                PREREQUISITE: you have to get a usearch licence. We recommand the 64-bit version (need to pay for)
                INSTALLATION: check Usearch section

## Run output directory

```bash
runs
└── <SEQNAME>-<RUNHASH>
    ├── logs
    ├── work
    ├── singularity
    └── results
```

- RUNHASH is computed with sequence file content en config file content (command line are not taken into account)
- `singularity` directory contains aumatically retrieved containers (from <https://lipm-browsers.toulouse.inra.fr/pub/singularity-repository/egnep/$VERSION/> )
- `work` directory contains task directories - WARNING : can become huge in size
- `results` directory contains final files

## Results output subdirectories

| Directory        | Content                                                                                                                             |
|------------------|-------------------------------------------------------------------------------------------------------------------------------------|
| 01_cfg              | Run and intermediate configuration files                                                                                         |
| 02_egn_param        | EuGene IMM and parameter files                                                                                                   |
| 03_db               | Species repeat sequences  and masked genome files                                                                                |
| 04_statistics       | Transcripts mapping summaries and gene prediction statistics                                                                     |
| 05_evidences        | GFF3 files of transcript mappings, protein alignments, ncRNA and repeat regions ab initio predictions used by EuGene             |
| 06_final_annotation | input genome file, complete genome annotation (GFF3 formatted) and feature sequence files (genes, mRNA, ncRNA, CDS and proteins) |
| 07_busco            | busco analyses at genome level, at protein level (based on annotation) and differences between                                   |
| 08_pipeline_info    | Execution timeline, resources usage, softwaresersions                                                                            |
| [99_workdir]        | By chromosome/scaffold evidences and prediction files (only available when --publish_workdir is set )                            |

## Results output files

```bash
$outdir
├── 01_cfg
│   ├── est1000.cfg
│   ├── est.upd0.cfg
│   └── run.parameters.cfg
├── 02_egn_param
│   ├── annotationV0.par
│   ├── annotationV1.par
│   └── eugene.mat
├── 03_db
│   ├── $output_prefix.allrepeat.fasta
│   ├── $output_prefix.masked.fasta
│   └── $output_prefix.repeat.fasta
├── 04_statistics
|   ├── transcripts.mapping.summary.tsv
|   ├── $output_prefix.YYYYMMDD.general_statistics.xls
|   ├── $output_prefix.YYYYMMDD.statistics_per_gene.xls
|   ├── SpliceSites.noncan.cfg
|   └── SpliceSites.report.txt
├── 05_evidences
│   ├── $output_prefix.blast{W}.gff3.gz
│   ├── $output_prefix.est{X}.gff3.gz
│   ├── $output_prefix.est{X}.rm_unspliced_10000000.gff3.gz
│   ├── $output_prefix.est1000.gff3.gz
│   ├── $output_prefix.est1000.rm_unspliced_10000000.gff3.gz
│   ├── $output_prefix.ltrharvest.gff3.gz
│   ├── $output_prefix.red.gff3.gz
│   ├── $output_prefix.repeat_noexpressed_nosimprot.gff3.gz
│   ├── $output_prefix.barrnap.gff3.gz
│   ├── $output_prefix.rfamscan.gff3.gz
│   └── $output_prefix.trnascan.gff3.gz
├── 06_final_annotation
│   ├── $output_prefix.YYYYMMDD.genome.fasta         #  input  genome sequence
│   ├── $output_prefix.YYYYMMDD.gff3             #  complete genome annotation
│   ├── $output_prefix.YYYYMMDD.gene.fasta          #  annotated gene sequences
│   ├── $output_prefix.YYYYMMDD.mrna.fasta          #  annotated mRNA sequences
│   ├── $output_prefix.YYYYMMDD.ncrna.fasta          #  annotated ncRNA sequences
│   ├── $output_prefix.YYYYMMDD.cds.fasta          #  annotated CDS sequences
│   └── $output_prefix.YYYYMMDD.prot.fasta          #  annotated protein sequences
├── 07_busco 
│   ├── genome_mode_$busco_lineage_dataset         # busco analysis genome based
│   ├── proteome_mode_$busco_lineage_dataset        # busco analysis annotation based
│   ├── summary.busco.$busco_lineage_dataset.bed       # busco regions found at genome level and missed by annotation 
│   └── summary.busco.$busco_lineage_dataset.tsv       # busco genome/proteome differences summary 
├── 08_pipeline_info
│   ├── softwares_info.tsv
│   ├── execution_report.html
│   ├── execution_timeline.html
│   ├── execution_trace.txt
│   └── pipeline_dag.svg
└── 99_workdir
    ├── entries.fof.sequences
    └── sequences
        ├── Seqid_A
        └── Seqid_B

```

## Usearch

As USEARCH license does not allow re-distribution of the binary, if you want to use USEARCH in your EGN-EP pipeline (--usearch parameter), you have build the blast-usearch container by yourself. The you will have to make this container available to your nextflow pipeline (--usearch_container_path)

To build this container

1. download build sources here: <https://lipm-browsers.toulouse.inra.fr/pub/singularity-repository/egnep/current/blast-usearch-singularity-src.tgz>
2. Follow `BUILD.md` instructions

WARNING: Set the correct usearch version in `conf/tools/versions.config`

## Contact

eugene-help@groupes.renater.fr

## Credits

- Sébastien Carrere
- Jérôme Gouzy
- Erika Sallet
- Thomas Schiex

## Citations

Sallet E., Gouzy J., Schiex T. (2019) EuGene: An Automated Integrative Gene Finder for Eukaryotes and Prokaryotes. Methods in Molecular Biology
