nextflow.enable.dsl=2

process EGNPCORE_changeGff3CoordinatesBatch {
    
    tag ("${source}:${seqid}")

    input:
    tuple val(seqid), val(source), path(gff3Files)

    output:
    tuple val (seqid), val(source), path("${seqid}.${source}.unsort.gff3")

    script:
    
    def filetype = 'gff3'
    def ext = 'gff3'
    def emptyHeader = '##gff-version 3'

    """
	batch_relocate.pl  --indir \$PWD --pattern '${seqid}:*-*.${source}.${ext}' --outdir \$PWD --filetype ${filetype}
    merge_${filetype}_files.pl --indir \$PWD --pattern '${seqid}:*-*.${source}.${ext}.relocate' --outfile ${seqid}.${source}.unsort.${ext}
    [ -s ${seqid}.${source}.unsort.${ext} ] && touch ${seqid}.${source}.unsort.${ext} ||  echo  "${emptyHeader}" > ${seqid}.${source}.unsort.${ext}
    """
}

process EGNPCORE_changeBlastxCoordinatesBatch {
    
    tag ("${source}:${seqid}")

    input:
    tuple val(seqid), val(source), path(m8File)

	output:
    tuple val (seqid), val(source), path("${seqid}.${source}.unsort.m8")
    
    script:
    
    def filetype = 'blastxm8'
    def ext = 'm8'
    def emptyHeader = '#blastx-m8'
    
    """
	batch_relocate.pl  --indir \$PWD --pattern '${seqid}:*-*.${source}.${ext}' --outdir \$PWD --filetype ${filetype}
    merge_${filetype}_files.pl --indir \$PWD --pattern '${seqid}:*-*.${source}.${ext}.relocate' --outfile ${seqid}.${source}.unsort.${ext}
    [ -s ${seqid}.${source}.unsort.${ext} ] && touch ${seqid}.${source}.unsort.${ext} ||  echo  "${emptyHeader}" > ${seqid}.${source}.unsort.${ext}
    """
}



process EGNPCORE_justPublish {

	publishDir "${params.outdir}/99_workdir/sequences/${seqid}", mode: 'copy'
    tag("${seqid}")
	
	input:
	tuple val (seqid), path(infile)

	output:
    path(infile)
    
    script:
    """
	echo  -n ${seqid}
    """

}

process EGNPCORE_finalPublish
{
    publishDir "${params.outdir}/06_final_annotation/", mode: 'copy'
    
	input:
	tuple val(seqid), path(genome), path(gff)
	path(finalFastaGene)
	path(finalFastaNcrna)
	path(finalFastaMrna)
	path(finalFastaCds)
	path(finalFastaProt)

	output:
	path("${params.output_prefix}.????????.*")
	
    script:
    """
    DATE=`date '+%Y%m%d'`
	ln -s ${gff} ${params.output_prefix}.\$DATE.gff3
	ln -s ${genome} ${params.output_prefix}.\$DATE.genome.fasta
	ln -s ${finalFastaGene} ${params.output_prefix}.\$DATE.gene.fasta
	ln -s ${finalFastaNcrna} ${params.output_prefix}.\$DATE.ncrna.fasta
	ln -s ${finalFastaMrna} ${params.output_prefix}.\$DATE.mrna.fasta
	ln -s ${finalFastaCds} ${params.output_prefix}.\$DATE.cds.fasta
	ln -s ${finalFastaProt} ${params.output_prefix}.\$DATE.prot.fasta
    """
}

process EGNPCORE_splitGff3BySeqid {
    
    input:
    tuple val(name), path(file)
    path(completeGenome)
    val(suffix)
    
    output:
    path('entries.fof')

    script:

    """
    split_gff3_byseqid.pl --refseq ${completeGenome} --infile ${file} --outdir splitgff --suffix .${name}${suffix}.gff3
    find \$PWD/splitgff/*.${name}${suffix}.gff3 > entries.fof
    """
}

process EGNPCORE_extractCDSFromBlastx {

    publishDir "${params.outdir}/02_egn_param", mode: 'copy'
    input:
    path(trancripts)
    path(blastx_db)
    val(hsp_pci)
    val(hsp_evalue)
    val(hsp_length)
    val(hsp_min_nb)

    output:
    path("CDS.tsv")

    script:
    """
    egn_extractCDSFromBlastx.pl --transcripts ${trancripts} --blastx_db ${blastx_db} --outfile CDS.tsv  --hsp_min_nb ${hsp_min_nb} --hsp_pci ${hsp_pci} --hsp_evalue ${hsp_evalue} --hsp_length ${hsp_length}
    """

}

process EGNPCORE_filterHspsForExtractCDS {
    input:
    path(blastx_db)
    val(hsp_pci)
    val(hsp_evalue)
    val(hsp_length)

    output:
    path("${blastx_db}.filtered")

    script:
    """
    egn_filterHspsForExtractCDS.pl  --blastx_db ${blastx_db} --outfile ${blastx_db}.filtered  --hsp_pci ${hsp_pci} --hsp_evalue ${hsp_evalue} --hsp_length ${hsp_length}
    """
}


process EGNPCORE_CatGzip {
    input:
    val(name)
    path(infiles)

    output:
    path("${name}.gz")

    script:
    """
    cat ${infiles} | gzip -c > ${name}.gz
    """
}
process EGNPCORE_dumpEstCfg {
    publishDir "${params.outdir}/01_cfg", mode: 'copy'

    input:
    path(confs)

    output:
    path("est.upd0.cfg")

    script:
    """
    grep -h ^# ${confs}| tr -d '\\n' | sed 's/^#/est_list=/;s/#/ /g' > est.upd0.cfg
    echo -e "\\n" >> est.upd0.cfg
    cat ${confs} >>  est.upd0.cfg
    """
}


process EGNPCORE_PublishEvidences {
    tag("${source}")
    publishDir "${params.outdir}/05_evidences", mode: 'copy'
    
    input:
    tuple val(source), path(infiles)
    
    output:
    path("${name}.gz")

    script:
    name = "${params.output_prefix}.${source}.gff3"
    
    if (source == 'repeat_noexpressed_nosimprot')
    {
        """
        ((grep  -h '^#' *.${source}.BT.gff3 || true) | (grep -v '^###' || true) | sort -u && (grep -h -v '^#'  *.${source}.BT.gff3 | sort -k1,1 -k4n,4 --buffer-size=2G || true) | (grep intergenic_region || true) | (grep -P "\\t-\\t" || true) | sed 's/\\t-\\t/\\t.\\t/') | bgzip -c > ${name}.gz
        """
    }
    else
    {
        """
        ((grep  -h '^#' *.${source}*.gff3  || true) | (grep -v '^###' || true) | sort -u && (grep -h -v '^#' *.${source}*.gff3 | sort -k1,1 -k4n,4 --buffer-size=2G || true)) | bgzip -c > ${name}.gz
        """
    }
}

process EGNPCORE_BuildTrainingDataSets
{
    input:
    path(seq)
    path(gff3)
    
    output:
    path("training/${seq.baseName}.training*")
    path("${seq.baseName}.training.INTRON.fa")

    script:
    """
    build_training_datasets_from_gff3.pl  --sequence ${seq} --gff3 ${gff3} --outprefix training/${seq.baseName}.training --parent_feature EST_match --child_feature match_part --intron_file ${seq.baseName}.training.INTRON.fa ${params.build_training_dataset_param}
    """
}

process EGNPCORE_FastaTotrainIMM 
{
    input:
    path(infile)
  
    output:
    path("${infile.baseName}.tsv")

    script:
    """
    egn_FastaTotrainIMM.pl --infile ${infile} --outfile ${infile.baseName}.tsv
    """
}


process EGNPCORE_DetectNonCanSites
{
    publishDir "${params.outdir}/04_statistics", mode: 'copy'

    input:
    path(seq)
    path(trainingFiles)
    
    output:
    path("SpliceSites.report.txt")
    path("SpliceSites.noncan.cfg")

    script:
    """
    egn_detectNonCanonSites.pl --report SpliceSites.report.txt --outcfg SpliceSites.noncan.cfg --trainingroot ./${seq.baseName}.training \
    --max_noncansite_candidate_nb ${params.max_noncansite_candidate_nb} \
    --noncansite_required_percent ${params.noncansite_required_percent} 
    """
}

process EGNPCORE_BuildEGNEukParameterFile {

    publishDir "${params.outdir}/02_egn_param", mode: 'copy'
    
    cache 'deep'
    
    input:
    val(prefix)
    val(repeat_mode)
    val(strand)
    path(noncansites_cfg)
    path(allEstConf)
    path(egn_run_parameters)

    output:
    path("${prefix}.par")

    String options = ''
    
    if (params.skip_trnascan) { options += ' --skip_trnascan' }
    if (params.skip_rfamscan) { options += ' --skip_rfamscan' }
    if (params.skip_rnammer) { options += ' --skip_rnammer'  }
    if (params.skip_barrnap) { options += ' --skip_barrnap'  }
    if (params.skip_ncrna_detection) { options += ' --skip_ncrna_detection'  }

    script:
    """
    egn_BuildEGNEukParameterFile.pl \
    --cfg=${egn_run_parameters} --noncansites_cfg=${noncansites_cfg} --est_cfg=${allEstConf} \
    --eugene_version=${params.version.eugene} --organism='${params.organism}' --wam_dataset=${params.domain} \
    --min_length_short_unspliced=${params.gmap_MIN_LEN_SHORT_UNSPLICED} \
    --outfile=${prefix}.par --strand ${strand} --repeat_mode ${repeat_mode} \
    ${options}
    """
}

process EGNPCORE_ConcatFasta {

	publishDir "${params.outdir}/03_db", mode: 'copy'
    
    input:
    path(files)
    val(suffix)

    output:
    path("${params.output_prefix}.${suffix}.fasta")

    script:
    """
    cat ${files} > ${params.output_prefix}.${suffix}.fasta
    """
}

process EGNPCORE_ConcatGff3 {

	input:
    tuple val(name), path(files)
    val(outfile)

    output:
    tuple val(name), path("${outfile}.gff3")

    script:
    """
    cat ${files} > ${outfile}.gff3
    """
}

process EGNPCORE_ConcatFinalGff3 {

	publishDir "${params.outdir}/06_final_annotation", mode: 'copy'
    input:
    path(files)

    output:
    tuple val("${params.output_prefix}"),path("final/${params.output_prefix}.gff3")

    script:
    """
    mkdir -p final
    cat  *.gff3 > final/${params.output_prefix}.gff3
    """
}

process EGNPCORE_selectTranscripts {

    input:
    tuple val(seqid), path(gff3)
    val(pcs)
    val(pci)

    output:
    path("${gff3.baseName}.transcripts.ids")
    tuple val('selected'), path("${gff3.baseName}.selected.transcripts.gff3")

    script:
    """
    egn_selectTranscripts.pl  --gff3 ${gff3} --training_est_pcs ${pcs} --training_est_pci ${pci} --outfile ${gff3.baseName}.transcripts.ids --filtered_gff3 ${gff3.baseName}.selected.transcripts.gff3
    """
}

process EGNPCORE_ConcatUniqueTxt {

    input:
    path(files)
    path(genome)
    val(suffix)

    output:
    path("${genome.baseName}.${suffix}.txt")

    script:
    """
    cat ${files} | sort -u > ${genome.baseName}.${suffix}.txt
    """
}

 process EGNPCORE_MaskSequence {
     
    input:
    tuple (val(seqid), path(seqin), path(gff3))
    val(source)

    output:
    tuple val(seqid), path("${seqid}.${source}.masked")

    script:
    if (gff3 == null)
    """
    cat ${seqin} > ${seqid}.${source}.masked
    """

    else
    """
    egn_maskSequence.pl --gff3 ${gff3} --seqin ${seqin} --seqout ${seqid}.${source}.masked
    """
 }

 process EGNPCORE_mergeGff3 {

    input:
    tuple val(seqid), path(sequence), path(gff3files)
    val (suffix)
    val (code)
    val (stranded)
    val (margin)
    val (maxlen)

    output:
    tuple val(seqid), path("${seqid}.${suffix}.gff3")

    script:
    """
    egn_mergeGFF3.pl --workdir \$PWD --indir \$PWD --sequence ${sequence} \
    --outfile ${seqid}.${suffix}.gff3 \
    --code ${code} \
    --stranded ${stranded} \
    --margin ${margin} \
	--maxlen ${maxlen}
    """
 }

 process EGNPCORE_setTypeGFF3 {
    input:
    tuple val(seqid), path(infile)
    val (typelist)
    val (both_strand)
    
    output:
    tuple val(seqid), path("${seqid}.typed.gff3")

    script:
    """
    egn_setTypeGFF3.pl --infile ${infile} --outfile ${seqid}.typed.gff3 --typelist '${typelist}' --both_strand ${both_strand}
    """
 }


 process EGNPCORE_splitByStrand {
    
    tag("${seqid} - ${suffix}")

    input:
    tuple val (seqid), path (gff3file)
    val (suffix)

    output:
    tuple val (seqid), path ("${outprefix}*.gff3")
    
    script:
    outprefix = "${seqid}.${suffix}"

    independent_strand_annotation = "touch ${outprefix}.BT.gff3"
    if (params.independent_strand_annotation) {
        independent_strand_annotation = "egn_splitbystrand.pl --gff3 ${outprefix}.BT.gff3 && mv ${outprefix}.BT.FW.gff3 ${outprefix}.FW.gff3 && mv ${outprefix}.BT.RV.gff3 ${outprefix}.RV.gff3 "
    }

    cmd = "sort -k1,1 -k4n,4 ${gff3file} > ${outprefix}.BT.gff3"
    
    """
    ${cmd}
    ${independent_strand_annotation}
    """
}

process EGNPCORE_CDSToTrainingFile {
    
    publishDir "${params.outdir}/02_egn_param", mode: 'copy'
    
    input:
    path(cds)
    val(hsp_min_nb)

    output:
    path("CDS.tsv")
    
    script:
    """
    egn_gmapCDSToTrainingFile.pl --fasta ${cds} --out CDS.tsv --hsp_min_nb ${hsp_min_nb}
    """
}



process EGNPCORE_softwaresInfo {
	output:
	path("egnpcore.softwares_info.tsv")

	script:
	"""
	softwares_info > egnpcore.softwares_info.tsv
	"""
}
