nextflow.enable.dsl=2

process GMAP_index {
    storeDir 'genomeIndex'
    label "process_high_memory"

    input:
    path(completeGenome)
    val(genomeMode)
    val(genomeHash)
    
    output:
    path(indexdir)
    //tuple val(estConf.name), path("${estConf.name}.*")
    
    script:
    indexdir = "${completeGenome.baseName}-${genomeHash}_gmapIndex-${params.version.gmap}"
    if (genomeMode == 'big')
    """
    mkdir -p ${indexdir}
    gmap_build -q 2 --build-sarray=0 -d ${completeGenome} -D ${indexdir} ${completeGenome} 2> ${indexdir}/gmap_idx.stde
    """
    else
    """
    mkdir -p ${indexdir}
    gmap_build -q 2 -d ${completeGenome} -D ${indexdir} ${completeGenome} 2> ${indexdir}/gmap_idx.stde
    """
}


process GMAP_map {
    label "process_high"
    
    input:
    path(completeGenome)
    path(genomeIndex)
    val(genomeMode)
    val(genomeHash)
    tuple val(estName), val(estConf), path(estSplitFile)
    
    
    output:
    tuple val(estName), path("${outfile}.filtered.gff3") 
    tuple val(estName), path("${outfileRmSpliced}.filtered.gff3")  optional true
    tuple val(estName), path("${outfile}.gmap_raw.gff3.gz")

    script:
    
    prg_gmap='gmap'
    if (genomeMode == 'big')
        prg_gmap='gmapl'
    else
        prg_gmap='gmap'
    
    outfile = "${completeGenome.baseName}.${estSplitFile}"
    outfileRmSpliced = "${completeGenome.baseName}.${estSplitFile}.rm_unspliced_${params.gmap_MIN_LEN_SHORT_UNSPLICED}"

    if (estConf.remove_unspliced as Integer > 0)
    
    """
    get_Est_Gmap.pl --mrnadb ${estSplitFile} --sequence \$PWD/${completeGenome} --pcs ${estConf.pcs} --pci ${estConf.pci} --prg_gmap=${prg_gmap} \
    --prg_gmap_param='${params.gmap_param}' --gmapbuild_dir=${genomeIndex} --min_length_short_unspliced ${params.gmap_MIN_LEN_SHORT_UNSPLICED} \
    --outfile=\$PWD/${outfile} \
    --nb_threads=${task.cpus}
    [ -s ${outfile}.gff3 ] && gmap_remove_aln.pl --in ${outfile}.gff3 --out ${outfile}.filtered.gff3  --min_exon_len=${params.gmap_filter_min_exon_len} --max_short_exon_number=${params.gmap_filter_max_short_exon_number}  || echo '##gff-version 3' >  ${outfile}.filtered.gff3
    [ -s ${outfileRmSpliced}.gff3 ] && gmap_remove_aln.pl --in ${outfileRmSpliced}.gff3 --out ${outfileRmSpliced}.filtered.gff3  --min_exon_len=${params.gmap_filter_min_exon_len} --max_short_exon_number=${params.gmap_filter_max_short_exon_number}  || echo '##gff-version 3' >  ${outfileRmSpliced}.filtered.gff3
    """
    
    else
    
    """
    get_Est_Gmap.pl --mrnadb ${estSplitFile} --sequence \$PWD/${completeGenome} --pcs ${estConf.pcs} --pci ${estConf.pci} --prg_gmap=${prg_gmap} \
    --prg_gmap_param='${params.gmap_param}' --gmapbuild_dir=${genomeIndex}  \
    --outfile=\$PWD/${outfile} \
    --nb_threads=${task.cpus}
    [ -s ${outfile}.gff3 ] && gmap_remove_aln.pl --in ${outfile}.gff3 --out ${outfile}.filtered.gff3  --min_exon_len=${params.gmap_filter_min_exon_len} --max_short_exon_number=${params.gmap_filter_max_short_exon_number}  || echo '##gff-version 3' >  ${outfile}.filtered.gff3
    """
}

process GMAP_updConf {

    publishDir "${params.outdir}/01_cfg/", mode: 'copy'

    input:
    path(infiles)
    
    output:
    path("est${params.allestnb}.cfg")

    script:
    """
    echo "est_list=${params.allestnb}" > est${params.allestnb}.cfg
    grep "est_${params.allestnb}_" `ls ${infiles} | head -1` >> est${params.allestnb}.cfg
    """
}

process GMAP_postProcessing {
    label "process_low"

    input:
    tuple val(seqid), path(estFiles)
    path(estCfgUpd0File)

    output:
    tuple val(seqid), path("results/${seqid}.*.gff3")
    path("results/${seqid}.report")
    path("results/${seqid}.est${params.allestnb}.cfg")
    
    script:

    gmap_intron_filter = ""
    if (params.gmap_intron_filter) {
        gmap_intron_filter = " --gmap_intron_filter"
    }

    independent_strand_annotation = ""
    if (params.independent_strand_annotation) {
        independent_strand_annotation = " --independent_strand_annotation"
    }

    unique = ""
    if (params.gmap_unique_filter) {
        unique = " --unique "
    }

    """
    gmap_postProcessing.pl ${unique} --allest_remove_unspliced=${params.allest_remove_unspliced} --outdir \$PWD/results --allestnb ${params.allestnb} --seqid ${seqid} --cfg_out ${seqid}.est${params.allestnb}.cfg --cfg ${estCfgUpd0File} --report  ${seqid}.report ${gmap_intron_filter} ${independent_strand_annotation} --min_length_short_unspliced ${params.gmap_MIN_LEN_SHORT_UNSPLICED} --gmap_smallexons_minlen ${params.gmap_smallexons_minlen} --verbose
    """
}

process GMAP_concat {
    label "process_low"
    
    input:
    tuple val(estName), path(estSplitFile)
    val(suffix)
    
    
    output:
    tuple val(estName), path("${outfile}")

    script:
    outfile = "${estName}${suffix}.gff3"

    """
    cat *.${estName}.*.gff3  > ${outfile}
    [ -s ${outfile} ] && touch ${outfile} || echo '##gff-version 3' >  ${outfile}
    """
    
}

process GMAP_reportSummary {

    publishDir "${params.outdir}/04_statistics", mode: 'copy'
    
    input:
    path(reportFiles)

    output:
    path("transcripts.mapping.summary.tsv")

    script:
    """
    ((grep -h  ^# ${reportFiles} | sort -u ) && grep -h -v ^# ${reportFiles} | sort -k2n,2 | sort -k1,1) > transcripts.mapping.summary.tsv
    """
}

process GMAP_softwaresInfo {
	output:
	path("gmap.softwares_info.tsv")

	script:
	"""
	softwares_info > gmap.softwares_info.tsv
	"""
}

