nextflow.enable.dsl=2

include { LIPMUTILS_splitEstDBs;LIPMUTILS_transfertGff3Attributes;LIPMUTILS_annotationStatistics;LIPMUTILS_fastaFilterIdsAndMinLenMulti;LIPMUTILS_splitSingleDB} from '../software/lipmutils/main.nf'
include { EGNPCORE_ConcatFasta;EGNPCORE_ConcatGff3;EGNPCORE_filterHspsForExtractCDS; EGNPCORE_CDSToTrainingFile;EGNPCORE_ConcatFinalGff3;EGNPCORE_finalPublish;EGNPCORE_justPublish;EGNPCORE_splitByStrand;EGNPCORE_setTypeGFF3;EGNPCORE_mergeGff3 as EGNPCORE_mergePreserveGff3;EGNPCORE_mergeGff3 as EGNPCORE_mergeRepeatGff3;EGNPCORE_ConcatUniqueTxt;EGNPCORE_selectTranscripts as EGNPCORE_selectTranscriptsPerSlice;EGNPCORE_selectTranscripts;EGNPCORE_BuildEGNEukParameterFile;EGNPCORE_CatGzip as EGNPCORE_CatGzipFilteredTranscripts; EGNPCORE_CatGzip;EGNPCORE_extractCDSFromBlastx;EGNPCORE_BuildTrainingDataSets;EGNPCORE_DetectNonCanSites;EGNPCORE_FastaTotrainIMM }  from '../software/egnpcore/main.nf'
include { EGNPCORE_BuildEGNEukParameterFile as EGNPCORE_BuildEGNEukParameterFileFW;EGNPCORE_BuildEGNEukParameterFile as EGNPCORE_BuildEGNEukParameterFileRV } from '../software/egnpcore/main.nf'
include { EUGENE_trainIMM;EUGENE_run; EUGENE_run as EUGENE_runFW; EUGENE_run as EUGENE_runRV;EUGENE_runBigGenome; EUGENE_runBigGenome as EUGENE_runBigGenomeFW; EUGENE_runBigGenome as EUGENE_runBigGenomeRV  }  from '../software/eugene/main.nf'
include { BLASTX_alignForTraining ; BLAST_makeblastdb}  from '../software/blast/main.nf'
include { GMAP_index;GMAP_map;GMAP_concat;GMAP_concat as GMAP_concatFiltered }  from '../software/gmap/main.nf'
include { BIOFILECONVERTER_gff2fasta;BIOFILECONVERTER_sortGff3;BIOFILECONVERTER_sortGff3Final } from '../software/biofileconverter/main.nf'
include { BEDTOOLS_subtract } from '../software/bedtools/main.nf'
include { BUSCO_protein;BUSCO_genome;BUSCO_summary } from '../software/busco/main.nf'
include { EGNPCORE_PublishEvidences as EGNPCORE_PublishEvidences_repeat_noexpressed_nosimprot } from '../software/egnpcore/main.nf'
include { CDHIT_trainingCDS } from '../software/cdhit/main.nf'
	
workflow trainEuGene {
	take:
		completeGenome
		configMap
		genomeMode
		genomeHash
		rawGmap4Training
		gmapIdx
		trainingSplitEstDBch

	main:

		minlen = params.hsp_training_length as Integer * 3
		Utils.info("Reuse ${configMap.train_est_db.name} mapping results for training",params.verbose)
		(selectedTrancriptsPerSlice,filteredGff3PerSlice) = EGNPCORE_selectTranscriptsPerSlice(rawGmap4Training,params.training_est_pcs,params.training_est_pci)
		mergedSlicedFilteredGff3Tuple = EGNPCORE_ConcatGff3(filteredGff3PerSlice.groupTuple(), 'mergedSlicedSelectedTranscripts')
		(selectedTrancriptsIds,filteredGff3) = EGNPCORE_selectTranscripts(mergedSlicedFilteredGff3Tuple,params.training_est_pcs,params.training_est_pci)
		
		selectedTrancriptsIds.first().map { t -> 
			lines = t.countLines()
			if (lines < params.training_min_est_mapped ) {	
				Utils.error("Not enough training est mapped to ensure good training (only ${lines} < ${params.training_min_est_mapped}). Ensure only the filtering process works")
			}
			
		}


		est_filter_file_ch = LIPMUTILS_fastaFilterIdsAndMinLenMulti(selectedTrancriptsIds.collect(), minlen,trainingSplitEstDBch.flatten())
		est_filter_file = EGNPCORE_ConcatFasta(est_filter_file_ch.collect(),'splitselect')
		est_filter_split = LIPMUTILS_splitSingleDB("${configMap.train_est_db.name}.selected", est_filter_file.collect(), params.hsp_training_splitsize).flatten()
			
		if (params.training_use_gmap_cds) {
			//TODO
			extractTuple = filteredGff3.first().combine(completeGenome).map{ t -> tuple(t.get(0),t.get(2),t.get(1)) }
			(trainingFastaGene, trainingFastaNcrna, trainingFastaMrna, trainingFastaCds, trainingFastaProt ) = BIOFILECONVERTER_gff2fasta(extractTuple)
			trainingFastaCdsNR = CDHIT_trainingCDS(trainingFastaCds)
			cdsFile = EGNPCORE_CDSToTrainingFile(trainingFastaCdsNR,params.hsp_training_min_nb)
			//BIOFILECONVERTER_gff2fasta()
			//RECUPERER LES CDS ET PROT AVEC BFC DEPUIS	filteredGff3 ?
			//RENVOYER	LES PROT POUR FAIRE UN NOUVEAU BLAST PROTECTED
			//UTILISER LES CDS POUR IMM

		}
		else {
			training_db_index = BLAST_makeblastdb(configMap.train_prot_db.file, 'prot', '')
			m8_files = BLASTX_alignForTraining(est_filter_split,configMap.train_prot_db.file,training_db_index.collect())
			//FILTER M8 FILES
			filtered_m8_files = EGNPCORE_filterHspsForExtractCDS(m8_files,params.hsp_training_pci,params.hsp_training_evalue,params.hsp_training_length)
			training_blastxfile = EGNPCORE_CatGzip('training_blastx.m8',filtered_m8_files.collect())
			cdsFile = EGNPCORE_extractCDSFromBlastx(est_filter_file,training_blastxfile,params.hsp_training_pci,params.hsp_training_evalue,params.hsp_training_length,params.hsp_training_min_nb)
		}


		
		conf = configMap.est.find {element -> element.name == configMap.train_est_db.name}
		splitEstDBsCh = est_filter_split.flatten().map{ 
		n -> {
				splitfile = n
				basename = splitfile.baseName
				tuple(configMap.train_est_db.name, conf, splitfile)
			}
		}
		(estGff3, estRmSplicedGff3) = GMAP_map(completeGenome.collect(),gmapIdx.collect(), genomeMode, genomeHash, splitEstDBsCh)
		estCatGff3 = GMAP_concat(estGff3.groupTuple(), '')
		(trainingFiles, intronFasta) = EGNPCORE_BuildTrainingDataSets(completeGenome,estCatGff3.flatten().last())
		(report, noncan_cfg) = EGNPCORE_DetectNonCanSites(completeGenome,trainingFiles)
		intronFile = EGNPCORE_FastaTotrainIMM(intronFasta)
		matrix = EUGENE_trainIMM(cdsFile,intronFile)

	emit:
		matrix
		noncan_cfg
}

workflow buildEGNRepeatFile{

	take:
	configMap
	pseudomolecules
	repeatEvidences
	ncRnaEvidences
	estEvidences
	proteinEvidences
	

	main:
	pseudomoleculesTuples = pseudomolecules.map{ it -> tuple (it.trim().split('/').last(),it.trim())}
	
	allrepeatregionsTuples = EGNPCORE_mergeRepeatGff3(pseudomoleculesTuples.join(repeatEvidences), 'allrepeatregions', 'repeat_region', 0, 0,-1)
	
	if (allrepeatregionsTuples) {
		Utils.info("Extract repeated regions", params.verbose)
		Utils.info("Repeat region selection", params.verbose)
	}
	
	// get est, blastx and ncRNA predictor result files
	// these regions has to be substracted to repeat regions
	estPreservedDBs =  configMap.est_preserved
	preservedEsts = estEvidences.map { t -> 
			t.get(1).flatten().findAll { file ->
				estPreservedDBs.any { file.name.contains("." + it + ".gff3" ) }
			}
		}.flatten().map{ it -> tuple (it.name.split('\\.').first(),it)}

	proteinPreservedDBs =  configMap.protein_preserved
	preservedProteins = proteinEvidences.map { t -> 
			t.get(1).flatten().findAll { file ->
				proteinPreservedDBs.any { file.name.contains("." + it + ".gff3" ) }
			}
		}.flatten().map{ it -> tuple (it.name.split('\\.').first(),it)}
	
	ncRnaDBs = ['barrnap','trnascan','rfamscan']

	preservedNcrna = ncRnaEvidences.map { t ->
		t.get(1).flatten().findAll { file ->
				ncRnaDBs.any { file.name.contains("." + it + ".BT.gff3" ) }
			}
		}.flatten().map{ it -> tuple (it.name.split('\\.').first(),it)}
	
	preservedEvidences = preservedEsts.concat(preservedProteins).concat(preservedNcrna).groupTuple()

	if (preservedEvidences) {
		Utils.info("Preserved region selection", params.verbose);
	}
	allpreservTuples = EGNPCORE_mergePreserveGff3(pseudomoleculesTuples.join(preservedEvidences), 'REPEAT.est_blastx_topreserv', 'repeat_region', 0, 100,-1)
	okrepeatfileTuples = BEDTOOLS_subtract(allrepeatregionsTuples.join(allpreservTuples), false)
	tmpOkrepeatfile4egnTuples = EGNPCORE_setTypeGFF3(okrepeatfileTuples, 'intergenic_region|intron', 2)
	okrepeatfile4egnTuples = EGNPCORE_splitByStrand(tmpOkrepeatfile4egnTuples,'repeat_noexpressed_nosimprot')
	if (params.publish_workdir)
	{
		EGNPCORE_justPublish(okrepeatfile4egnTuples)
	}
	//repeat_noexpressed_nosimprotFiles = okrepeatfile4egnTuples.collect().map { t -> t.get(1) }.flatten().filter{ ! it.name.contains('.RV.') && ! it.name.contains('.FW.')}.map{ f -> tuple('repeat_noexpressed_nosimprot', f)}.groupTuple()
	EGNPCORE_PublishEvidences_repeat_noexpressed_nosimprot(filterEvidenceFiles(okrepeatfile4egnTuples,'repeat_noexpressed_nosimprot'))
	

	emit:
	okrepeatfile4egnTuples
	
}
	
workflow launchAllEGN {

	take:
	pseudomolecules
	matrix
	noncan_cfg
	evidencesTuples
	allEstConf
	run_parameters
	genomeMode

	main:

	pseudomoleculesTuples = pseudomolecules.map{ it -> tuple (it.trim().split('/').last(),it.trim())}
	sourceTuples = pseudomoleculesTuples.combine(evidencesTuples, by: 0)

	def repeatmode = 1
	if (params.skip_repeat_masking) {
		repeatmode = 0
	}

	if (! params.independent_strand_annotation)	{
		annotationV1_param = EGNPCORE_BuildEGNEukParameterFile('annotationV1',repeatmode,0,noncan_cfg,run_parameters)
		if (genomeMode == 'big')
		{
			annotationV1gff3Tuples = EUGENE_runBigGenome(annotationV1_param.collect(), matrix.collect(), sourceTuples, 'ANNOT')
		}
		else
		{
			annotationV1gff3Tuples = EUGENE_run(annotationV1_param.collect(), matrix.collect(), sourceTuples, 'ANNOT')
		}
		
	}
	else {
		Utils.info("NOT IMPLEMENTED", params.verbose)
		annotationV1FW_param = EGNPCORE_BuildEGNEukParameterFileFW('annotationV1',repeatmode,1,noncan_cfg,allEstConf,run_parameters)
		annotationV1RV_param = EGNPCORE_BuildEGNEukParameterFileRV('annotationV1',repeatmode,-1,noncan_cfg,allEstConf,run_parameters)
		
		if (genomeMode == 'big')
		{
			annotationV1FWgff3Tuples = EUGENE_runBigGenomeFW(annotationV1FW_param.collect(), matrix.collect(), sourceTuples, 'ANNOT.FW')
			annotationV1RVgff3Tuples = EUGENE_runBigGenomeRV(annotationV1RV_param.collect(), matrix.collect(), sourceTuples, 'ANNOT.RV')
		}
		else 
		{
			annotationV1FWgff3Tuples = EUGENE_runFW(annotationV1FW_param.collect(), matrix.collect(), sourceTuples, 'ANNOT.FW')
			annotationV1RVgff3Tuples = EUGENE_runRV(annotationV1RV_param.collect(), matrix.collect(), sourceTuples, 'ANNOT.RV')
		}
		annotationV1gff3Tuples = BIOFILECONVERTER_sortGff3(annotationV1FWgff3Tuples.concat(annotationV1RVgff3Tuples).groupTuple())
	}

	emit:
	annotationV1gff3Tuples
}


workflow consolidateAnnotation {
	
	take:
	completeGenome
	annotationV1gff3Tuples
	ncRnaEvidences

	main:
	
	mergeTuples = annotationV1gff3Tuples
							.join(ncRnaEvidences.groupTuple(),remainder: true)
							.map{ t -> tuple (t.get(0), t.get(1), t.get(2).flatten())
							}
	gff3FilesTuples = LIPMUTILS_transfertGff3Attributes(mergeTuples)
	finalGff3Tuples = BIOFILECONVERTER_sortGff3Final(gff3FilesTuples)
	if (params.publish_workdir)
	{
		EGNPCORE_justPublish(finalGff3Tuples)
	}
	finalGff3 = EGNPCORE_ConcatFinalGff3(finalGff3Tuples.map{t -> t.get(1)}.flatten().collect())
	extractTuple = finalGff3.combine(completeGenome).map{ t -> tuple(t.get(0),t.get(2),t.get(1)) }
	(finalFastaGene, finalFastaNcrna, finalFastaMrna, finalFastaCds, finalFastaProt ) = BIOFILECONVERTER_gff2fasta(extractTuple)
	
	EGNPCORE_finalPublish(extractTuple, finalFastaGene, finalFastaNcrna, finalFastaMrna, finalFastaCds, finalFastaProt)
	LIPMUTILS_annotationStatistics(extractTuple)
	busco_protein_dir = BUSCO_protein(finalFastaProt)
	busco_genome_dir = BUSCO_genome(completeGenome)
	BUSCO_summary(busco_protein_dir,busco_genome_dir)


}


workflow runEugene {
	take:
		all_ok
		inputSequences
	
	main:
		EUGENE_run(all_ok, inputSequences)

}

def filterEvidenceFiles(intuples, source)
{
	outfiles = intuples.groupTuple().map { t -> t.get(1).flatten() }.flatten().filter{ ! it.name.contains('.RV.') && ! it.name.contains('.FW.')}.map{ f -> tuple(source, f)}.groupTuple()
	return 	outfiles
}
